package org.openstreetmap.josm.plugins.lanetool.data;

import java.util.Collection;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.Observable;
import java.util.Observer;
import java.util.Set;

import org.openstreetmap.josm.Main;
import org.openstreetmap.josm.command.AddCommand;
import org.openstreetmap.josm.command.Command;
import org.openstreetmap.josm.command.SequenceCommand;
import org.openstreetmap.josm.data.SelectionChangedListener;
import org.openstreetmap.josm.data.osm.OsmPrimitive;
import org.openstreetmap.josm.data.osm.Relation;
import org.openstreetmap.josm.data.osm.RelationMember;
import org.openstreetmap.josm.data.osm.Way;

/**
 * Represents the lane information of the currently selected way.
 */
public class LanesData extends Observable implements SelectionChangedListener, Observer {

	private Collection<? extends OsmPrimitive> selection;
	private LaneGroup laneGroup;
	private boolean canCreateLaneGroup = false;
	
	public void update(Observable o, Object arg) {
		update();		
	}
	
	private void update() {
		laneGroup = null;
		canCreateLaneGroup = false;
		
		Set<Relation> laneGroupRelations = new HashSet<Relation>();
		
		if (selection != null) {

			for (OsmPrimitive selectedPrimitive : selection) {

				if (selectedPrimitive instanceof Way && selection.size() == 1) {
					canCreateLaneGroup = true;
				}

				if (selectedPrimitive instanceof Relation
						&& "lane_group".equals(((Relation)selectedPrimitive).get("type"))) {

					laneGroupRelations.add((Relation)selectedPrimitive);						
					
				} else if (selectedPrimitive instanceof Way
						|| (selectedPrimitive instanceof Relation
								&& "lane".equals(((Relation)selectedPrimitive).get("type")))) {

					//check whether the selected primitive is member of a lane_group relation

					for (Relation relation : Main.main.getCurrentDataSet().getRelations()) {

						if ("lane_group".equals(relation.get("type"))) {

							boolean relationCorrect = relation.getMembers().contains(new RelationMember("highway", selectedPrimitive));

							for (RelationMember member : relation.getMembers()) {
								if (relationCorrect) {
									break;
								}
								if (member.getMember() == selectedPrimitive) {
									relationCorrect = true;
									try {
										Integer.parseInt(member.getRole());
									} catch (NumberFormatException nfe) {
										relationCorrect = false;
									}								
								}
							}


							if (relationCorrect) {
								laneGroupRelations.add(relation);
							}
						}

					}

				}
				
			}
			
		}
		
		if (laneGroupRelations.size() == 1) { //TODO: support more than one relation if they are "identical"
			laneGroup = new LaneGroup(laneGroupRelations.iterator().next());
			laneGroup.addObserver(this);
			canCreateLaneGroup = false;
		} else {
			laneGroup = null;
		}
		
		setChanged();
		notifyObservers();
		
	}
	
	public void selectionChanged(Collection<? extends OsmPrimitive> newSelection) {
		this.selection = newSelection;
		update();		
	}
	
	public boolean canCreateLaneGroup() {
		return canCreateLaneGroup;
	}
	
	public void createLaneGroup() {
		if (!canCreateLaneGroup) {
			throw new IllegalStateException("cannot create lane group now");
		}
		
		Collection<Command> commands = new LinkedList<Command>();
		
		for (OsmPrimitive primitive : selection) {
						
			Relation relation = new Relation();
			relation.addMember(new RelationMember("highway", primitive));
			relation.put("type", "lane_group");
			relation.setModified(true);
			
			AddCommand command = new AddCommand(relation);
			commands.add(command);
			
		}
		
		SequenceCommand sequenceCommand = new SequenceCommand("create root lane", commands);
		sequenceCommand.executeCommand();
		Main.main.getCurrentDataSet().fireSelectionChanged(); //indirectly also causes update() call
				
	}

	public LaneGroup getLaneGroup() {
		return laneGroup;
	}
	
}
