package org.openstreetmap.josm.plugins.lanetool.data;

import java.util.ArrayList;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

import org.openstreetmap.josm.Main;
import org.openstreetmap.josm.command.AddCommand;
import org.openstreetmap.josm.command.ChangeCommand;
import org.openstreetmap.josm.command.Command;
import org.openstreetmap.josm.command.DeleteCommand;
import org.openstreetmap.josm.command.SequenceCommand;
import org.openstreetmap.josm.data.osm.Node;
import org.openstreetmap.josm.data.osm.OsmPrimitive;
import org.openstreetmap.josm.data.osm.Relation;
import org.openstreetmap.josm.data.osm.RelationMember;
import org.openstreetmap.josm.data.osm.Way;
import org.openstreetmap.josm.gui.dialogs.relation.RelationEditor;

public class RelationLane extends AbstractLane<Relation> {
	
	RelationLane(Relation osmObject, LaneGroup parent) {
		super(osmObject, parent);
		
		assert "lane".equals(osmObject.get("type"));
				
	}

	@Override
	public void setTag(String key, String value) {
		
		Relation oldRelation = new Relation();
		oldRelation.cloneFrom(osmObject);
		
		osmObject.put(key, value);
		
		Command command = new ChangeCommand(oldRelation, osmObject);
		command.executeCommand();
		Main.main.getCurrentDataSet().fireSelectionChanged();
		
		setChanged();
		notifyObservers();
	}

	@Override
	public void removeTag(String key) {
		
		Relation oldRelation = new Relation();
		oldRelation.cloneFrom(osmObject);
		
		osmObject.remove(key);
		
		Command command = new ChangeCommand(oldRelation, osmObject);
		command.executeCommand();
		Main.main.getCurrentDataSet().fireSelectionChanged();
		
		setChanged();
		notifyObservers();
	}
	
	/** let the user edit this lane's relation */
	public void editRelation() {		
		RelationEditor.getEditor(Main.main.getEditLayer(), osmObject, new ArrayList<RelationMember>(0)).setVisible(true);		
	}
	
	/**
	 * creates a way-based lane with identical tags
	 * and uses it to replace this lane in the lane group
	 */
	public void transformToWay() {
		
		if (parent.getHighway() != null) {
			
			Collection<Command> commands = new LinkedList<Command>();
			
			/* create clone of highway way with tags from the relation */
			
			Way newWay = new Way();
					
			for (Node node : parent.getHighway().getNodes()) {
				
				Node newNode = new Node(node.getCoor()); 
				newNode.setModified(true);
				
				newWay.getNodes().add(newNode);
				
				AddCommand addNodeCommand = new AddCommand(newNode);
				commands.add(addNodeCommand);
				
			}
			
			for (String key : osmObject.keySet()) {
				newWay.put(key, osmObject.get(key));
			}
			
			newWay.setModified(true);	
			
			AddCommand addWayCommand = new AddCommand(newWay);
			commands.add(addWayCommand);
			
			/* modify all referencing relations to reference the way */
			
			for (Relation r : Main.main.getCurrentDataSet().getRelations()) {
				Relation oldR = new Relation();
				oldR.cloneFrom(r);
				Collection<RelationMember> membersToRemove = new ArrayList<RelationMember>();
				Collection<RelationMember> membersToAdd = new ArrayList<RelationMember>();
				for (RelationMember member : r.getMembers()) {
					if (member.getMember() == osmObject) {
						membersToRemove.add(member);
						RelationMember newMember = new RelationMember(member.getRole(), newWay);
						membersToAdd.add(newMember);
					}
				}
				if (membersToRemove.size() > 0) {
					List<RelationMember> memberList = r.getMembers();
					memberList.removeAll(membersToRemove);
					memberList.addAll(membersToAdd);
					r.setMembers(memberList);
					r.setModified(true);
					Command changeCommand = new ChangeCommand(oldR, r);
					commands.add(changeCommand);
				}
			}
			
			/* delete the relation */
			
			DeleteCommand deleteCommand = new DeleteCommand(osmObject);
			commands.add(deleteCommand);
					
			/* replace relation with way in selection */
			
			Collection<OsmPrimitive> selection = Main.main.getCurrentDataSet().getSelected();
			
			if (selection.contains(osmObject)) {
				
				selection.remove(osmObject);
				selection.add(newWay);
				Main.main.getCurrentDataSet().setSelected(selection);
							
			}
			
			/* execute sequence command */
			
			SequenceCommand sequenceCommand = new SequenceCommand("replace lane's relation with way", commands);
			sequenceCommand.executeCommand();
					
			Main.main.getCurrentDataSet().fireSelectionChanged();
			
		}
			
	}
	
}
