package org.openstreetmap.josm.plugins.lanetool.view.style;

import java.awt.Color;
import java.awt.Image;
import java.io.IOException;

import javax.imageio.ImageIO;

import org.openstreetmap.josm.plugins.lanetool.data.Lane;

public class DefaultStyle implements LanesStyle {

	public Color getBackgroundColor() {
		return Color.BLACK;
	}
	
	public Image getBackgroundImage() {
		
		return loadImage("images/background/bg.png");
	}

	public Image getLaneImage(Lane lane) {
		
		if (lane == null) {
			
			return loadImage("images/lanes/none.png");
			
		} else if (lane instanceof Lane) {
			String laneType = ((Lane)lane).getLaneType();
			
			if ("cycleway".equals(laneType)) {						
				return loadImage("images/lanes/cycleway.png");						
			} else if ("footway".equals(laneType)) {						
				return loadImage("images/lanes/footway.png");			
			} else if ("vehicle_lane".equals(laneType)) {						
				return loadImage("images/lanes/vehicle_lane.png");			
			} else if ("bus_lane".equals(laneType)) {						
				return loadImage("images/lanes/bus_lane.png");			
			}
		}
	
		return loadImage("images/lanes/unknown.png");

	}
	
	public Image getDirectionImage(Lane lane) {
		
		if (lane.getDirection() != null) {
			switch(lane.getDirection()) {
			case FORWARD: return loadImage("images/directions/forward.png");
			case BACKWARD: return loadImage("images/directions/backward.png");
			case BOTH: return loadImage("images/directions/both.png");
			default: throw new Error("unknown lane direction: " + lane.getDirection());
			}
		} else {
			return loadImage("images/directions/unknown.png");
		}		
		
	}
	
	private Image loadImage(String resourceIdentifier) {

		ClassLoader cldr = this.getClass().getClassLoader();
		
		java.net.URL imageURL = cldr.getResource(resourceIdentifier);
		
		try {
			return ImageIO.read(imageURL);
		} catch (IOException e) {
			e.printStackTrace();
			return null;
		}
		
	}
	
}
